#ifndef _GAS_MONITOR_
#define _GAS_MONITOR_

enum USB_PORT_E{

	USB_PORT_0 = 0,
	USB_PORT_1 = 1,
};

#define H2_THRESHOLD 120
#define CO_THRESHOLD 120
#define CO2_THRESHOLD 120
#define CH4_THRESHOLD 120
#define C2H2_THRESHOLD 120
#define C2H4_THRESHOLD 120
#define C2H6_THRESHOLD 120

//监测周期
void xSet_monitor_period(const char* mon_period);
void xSet_uart_buadrate(const char* uart_buadrate);
//气体告警阈值
void xSet_H2_threshold(const char* H2_threshold);
void xSet_CO_threshold(const char* CO_threshold);
void xSet_CO2_threshold(const char* CO2_threshold);
void xSet_CH4_period(const char* CH4_threshold);
void xSet_C2H2_threshold(const char* C2H2_threshold);
void xSet_C2H4_period(const char* C2H4_threshold);
void xSet_C2H6_threshold(const char* C2H6_threshold);
void xSet_H2_alert(const char* H2_alert);
void xSet_CO_alert(const char* CO_alert);
void xSet_CO2_alert(const char* CO2_alert);

int xGas_alert_task_init(char* uart_port);
#endif // !_GAS_MONITOR
